#include <stdio.h>
#include <conio.h>
#include <kolibrisys.h>

// (c) Rustem Gimadutdinov aka rgimad. 2020

int main(int argc, char **argv)
{
    if (con_init_console_dll()) return 1; // init fail
    con_set_title("lspci v0.01a");

    dword pci_ver_info = _ksys_get_pci_version();
    byte pci_main_ver = (pci_ver_info & 0x0000FF00) >> 8;
    byte pci_sub_ver = pci_ver_info & 0x000000FF;

    printf("PCI version = %d.%d\n", pci_main_ver, pci_sub_ver);

    byte pci_last_bus = _ksys_get_last_pci_bus();
    if (pci_last_bus == 0xFF)
    {
	printf("[-] no pci bus found\n");
	return 1;
    }
    printf("Last PCI bus = %d\n\n", pci_last_bus);

    byte v_bus = 0;
    byte v_dev = 0;

    byte dev_func;
    byte dev_num;

    word pci_vendor_id;
    word pci_device_id;
    byte pci_rev;
    byte pci_class;
    byte pci_subclass;
    byte pci_interface;
    byte pci_irq;

    int index = 1;
    while (v_bus <= pci_last_bus)
    {
	//printf("v_bus = %x  v_dev = %x\n", v_bus, v_dev);
	dev_num = v_dev >> 3;
	dev_func = v_dev & 7;
	dword rcd = _ksys_pci_read_config_dword(v_bus, dev_num, dev_func, 0);
	word rcd_lword = rcd & 0x0000FFFF;
	if (rcd_lword != 0 && rcd_lword != 0xFFFF)
	{
	    pci_vendor_id = rcd_lword;
	    pci_device_id = rcd >> 16;

	    pci_rev = _ksys_pci_read_config_byte(v_bus, dev_num, dev_func, 0x08);
	    pci_class = _ksys_pci_read_config_byte(v_bus, dev_num, dev_func, 0x0b);
	    pci_subclass = _ksys_pci_read_config_byte(v_bus, dev_num, dev_func, 0x0a);
	    pci_interface = _ksys_pci_read_config_byte(v_bus, dev_num, dev_func, 0x09);
	    pci_irq = _ksys_pci_read_config_byte(v_bus, dev_num, dev_func, 0x3c);

	    printf("%d) ", index++);
	    printf("Vendor id = %x\n", pci_vendor_id);
	    printf("Device id = %x\n", pci_device_id);
	    printf("Bus# = %x\n", v_bus);
	    printf("Dev# = %x\n", dev_num); // get device number
	    printf("Fnc = %x\n", dev_func); // get function number
	    printf("Rev = %x\n", pci_rev);
	    printf("Class = %x\n", pci_class);
	    printf("Subclass = %x\n", pci_subclass);
	    printf("Interface = %x\n", pci_interface);
	    if (pci_irq == 0xFF)
	    {
		printf("IRQ = --\n");
	    } else
	    {
		printf("IRQ = %x\n", pci_irq);
	    }
	    printf("\n");
	}

	if (v_dev & 7 == 0)
	{
	    byte rcb = _ksys_pci_read_config_byte(v_bus, dev_num, dev_func, 0x0e);
	    if (rcb & 128 == 0) // if not a multifunctional device go to next device
	    {
		v_dev = v_dev | 7;
	    }
	}

	v_dev++;
	if (v_dev == 0)
	{
	    //v_dev = 0;
	    v_bus++;
	}
    }

    //printf("end");
    //printf("Vendor id of dev 0 = %x\n", vendor_id);

    con_exit(0);
    return 0;
}